"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var _1 = require("./");
var promClient = require("prom-client");
var supertest = require("supertest");
describe('@usvc/server', function () {
    describe('middleware hooks', function () {
        var server;
        describe('before', function () {
            before(function () {
                promClient.register.clear();
                server = _1.createServer({
                    middlewares: {
                        before: [
                            function (_req, res) {
                                res
                                    .status(418)
                                    .json('you\'re not getting past this teapot');
                            },
                        ],
                    },
                });
            });
            it('works', function () {
                return supertest(server)
                    .get('/')
                    .expect('x-powered-by', 'Express') // no http security
                    .expect(418);
            });
        });
        describe('after', function () {
            before(function () {
                server = _1.createServer({
                    middlewares: {
                        after: [
                            function (_req, res) {
                                res
                                    .status(418)
                                    .json('you\'re not getting past this teapot');
                            },
                        ],
                    },
                });
            });
            it('works', function () {
                return supertest(server)
                    .get('/')
                    .expect(418)
                    .then(function (_a) {
                    var header = _a.header;
                    chai_1.expect(header['x-powered-by'])
                        .to.not.exist; // http security applied
                });
            });
        });
    });
    // / middleware hooks
    describe('integration tests', function () {
        it('works', function () {
            chai_1.expect(function () {
                _1.createServer();
            }).to.not.throw();
        });
        context('with configuration', function () {
            var data = {
                a: 1, b: '2', c: true, d: {
                    A: 1, B: '2', C: false,
                },
            };
            var server;
            describe('cors', function () {
                context('urls', function () {
                    var errors = [];
                    before(function () {
                        server = _1.createServer({
                            cors: {
                                urls: ['localhost'],
                            },
                        });
                        server.get('/', function (_req, res) {
                            res.json('ok');
                        });
                        server.use(function (err, _req, res, _next) {
                            errors.push(err);
                            res
                                .status(err.status ? err.status : 500)
                                .json(err.message);
                        });
                    });
                    afterEach(function () {
                        errors = [];
                    });
                    it('can have custom allowed urls', function () {
                        return Promise.all([
                            supertest(server)
                                .get('/')
                                .set('Origin', 'remotehost')
                                .expect(401)
                                .then(function () { return true; }),
                            supertest(server)
                                .get('/')
                                .set('Origin', 'localhost')
                                .expect(200)
                                .then(function (response) {
                                chai_1.expect(response.header['access-control-allow-origin'])
                                    .to.deep.equal('localhost');
                                return true;
                            }),
                            supertest(server)
                                .options('/')
                                .set('Origin', 'localhost')
                                .expect(200)
                                .then(function (response) {
                                chai_1.expect(response.header['access-control-allow-origin'])
                                    .to.deep.equal('localhost');
                                chai_1.expect(response.header['access-control-allow-credentials'])
                                    .to.deep.equal('true');
                                chai_1.expect(response.header['access-control-allow-methods'])
                                    .to.deep.equal('GET,POST,PUT,PATCH,DELETE,OPTIONS,HEAD');
                                chai_1.expect(response.header['access-control-max-age'])
                                    .to.deep.equal('86400000');
                                return true;
                            }),
                        ]).then(function (results) {
                            results.forEach(function (result) {
                                chai_1.expect(result).to.deep.equal(true);
                            });
                            chai_1.expect(errors).to.have.length(1);
                            chai_1.expect(errors[0]).to.have.property('message');
                            chai_1.expect(errors[0].message).to.contain('remotehost');
                        });
                    });
                });
                // / urls
                context('preflight continue', function () {
                    var optionsSuccessStatus = 999;
                    var results;
                    before(function () {
                        return supertest(_1.createServer({
                            cors: {
                                optionsSuccessStatus: optionsSuccessStatus,
                                preflightContinue: false,
                            },
                        }))
                            .options('/')
                            .then(function (response) {
                            results = response;
                        });
                    });
                    it('returns the :optionsSuccessStatus value', function () {
                        chai_1.expect(results.status).to.deep.equal(optionsSuccessStatus);
                    });
                    it('does not disrupt other defaults', function () {
                        chai_1.expect(results.header['access-control-allow-credentials'])
                            .to.deep.equal('true');
                        chai_1.expect(results.header['access-control-allow-methods'])
                            .to.deep.equal('GET,POST,PUT,PATCH,DELETE,OPTIONS,HEAD');
                    });
                });
            });
        });
        // / with configuration
        context('with defaults', function () {
            var data = {
                a: 1, b: '2', c: true, d: {
                    A: 1, B: '2', C: false,
                },
            };
            var errors = [];
            var server;
            before(function () {
                server = _1.createServer();
                server.post('/', function (req, res) {
                    res.json(req.body);
                });
                server.get('/_cookies', function (req, res) {
                    res.json(req.cookies);
                });
                server.post('/_cookies', function (req, res) {
                    req.session.views = (req.session.views || 0) + 1;
                    res.json(req.session.views);
                });
                server.use(function (err, _req, res, _next) {
                    errors.push(err);
                    res
                        .status(err.status ? err.status : 500)
                        .json(err.message);
                });
            });
            afterEach(function () {
                errors = [];
            });
            it('is cors enabled', function () {
                return Promise.all([
                    supertest(server)
                        .options('/')
                        .expect(200)
                        .then(function (_a) {
                        var header = _a.header;
                        return ((header['access-control-allow-credentials'] !== undefined)
                            && (header['access-control-allow-methods'] !== undefined)
                            && (header['access-control-max-age'] !== undefined));
                    }),
                    supertest(server)
                        .options('/')
                        .set('Origin', 'http://localhost')
                        .expect(200)
                        .catch(function (error) { return true; }),
                ]).then(function (results) {
                    results.forEach(function (result) {
                        chai_1.expect(result).to.deep.equal(true);
                    });
                    chai_1.expect(errors).to.have.length(1);
                    chai_1.expect(errors[0]).to.have.property('message');
                    chai_1.expect(errors[0].message).to.contain('http://localhost');
                });
            });
            it('parses json body data', function () {
                return supertest(server)
                    .post('/')
                    .set('Content-Type', 'application/json')
                    .send(data)
                    .expect(200)
                    .then(function (_a) {
                    var body = _a.body;
                    chai_1.expect(body).to.deep.equal(data);
                });
            });
            it('parses url encoded body data', function () {
                return supertest(server)
                    .post('/')
                    .set('Content-Type', 'application/x-www-form-urlencoded')
                    .send(data)
                    .expect(200)
                    .then(function (_a) {
                    var body = _a.body;
                    chai_1.expect(body.a).to.deep.equal(data.a.toString());
                    chai_1.expect(body.b).to.deep.equal(data.b.toString());
                    chai_1.expect(body.c).to.deep.equal(data.c.toString());
                    chai_1.expect(body.d.A).to.deep.equal(data.d.A.toString());
                    chai_1.expect(body.d.B).to.deep.equal(data.d.B.toString());
                    chai_1.expect(body.d.C).to.deep.equal(data.d.C.toString());
                });
            });
            it('generates cookies', function () {
                return supertest(server)
                    .post('/_cookies')
                    .expect(200)
                    .then(function (_a) {
                    var header = _a.header;
                    return header['set-cookie'];
                })
                    .then(function (cookies) {
                    return supertest(server)
                        .post('/_cookies')
                        .set('Cookie', cookies)
                        .expect(200);
                })
                    .then(function (_a) {
                    var body = _a.body;
                    chai_1.expect(body).to.deep.equal(2);
                });
            });
            it('parses cookies', function () {
                return supertest(server)
                    .get('/_cookies')
                    .set('Cookie', 'a=1; b=c; d=true;')
                    .expect(200)
                    .then(function (_a) {
                    var body = _a.body;
                    chai_1.expect(body).to.deep.equal({
                        a: '1', b: 'c', d: 'true',
                    });
                });
            });
        });
        // / with defaults
    });
});
//# sourceMappingURL=index.test.js.map